/*******************************************************************************
 
  $Id: RegSupport.cpp,v 1.1 1997/10/05 06:38:52 eug Exp eug $

  (C)Copyright 1997 by Eugene Surovegin <ebs@glasnet.ru>

  Registry support

*******************************************************************************/
#include "stdafx.h"
#include <string.h>
#include <stdio.h>

static int add_to_list(const char*,char*,int);
static int remove_from_list(const char*,char*,int);
// Not use stricmp from std runtime - 20K overhead
static int StrIcmp(const char*, const char*);
static void report_error(HWND,DWORD,const char*);

// String constants
static char LsaSubKeyPath[]="SYSTEM\\CurrentControlSet\\Control\\Lsa";
static char LogSubKeyPath[]="SYSTEM\\CurrentControlSet\\Services\\EventLog\\Security\\DictionaryPasswordFilter";
static char LsaValueName[]="Notification Packages";
static char LsaAppValue[]="DictFilt";

bool InstallFilter(HWND hwnd){

  HKEY hk;
  bool res=false;
  char value[255];
  DWORD dwSize=sizeof(value),error;
  
  ZeroMemory(value,sizeof(value));

  if ((error=RegOpenKeyEx(HKEY_LOCAL_MACHINE,LsaSubKeyPath,0,KEY_ALL_ACCESS,&hk))==ERROR_SUCCESS){
    
    if ( (error=RegQueryValueEx(hk,LsaValueName,0,NULL,(BYTE*)value,&dwSize))==ERROR_SUCCESS){
      dwSize=add_to_list(LsaAppValue,(char*)value,dwSize);
      if ((error=RegSetValueEx(hk,LsaValueName,0,REG_MULTI_SZ,(BYTE*)value,dwSize))==ERROR_SUCCESS)
        res=true;
    }
    else if (error==ERROR_FILE_NOT_FOUND){
      strcpy(value,LsaAppValue);
      if ((error=RegSetValueEx(hk,LsaValueName,0,REG_MULTI_SZ,(BYTE*)value,strlen(value)+2))==ERROR_SUCCESS)
        res=true;
    }
  }
  if (!res)
    report_error(hwnd,error,"Error installing notification entry");

  return res;
}

bool RemoveFilter(HWND hwnd){
  
  HKEY hk;
  bool res=false;
  char value[255];
  DWORD dwSize=sizeof(value),error;

  ZeroMemory(value,sizeof(value));

  if ((error=RegOpenKeyEx(HKEY_LOCAL_MACHINE,LsaSubKeyPath,0,KEY_ALL_ACCESS,&hk))==ERROR_SUCCESS){
    if ( (error=RegQueryValueEx(hk,LsaValueName,0,NULL,(BYTE*)value,&dwSize))==ERROR_SUCCESS){
      dwSize=remove_from_list(LsaAppValue,(char*)value,dwSize);
      if ((error=RegSetValueEx(hk,LsaValueName,0,REG_MULTI_SZ,(BYTE*)value,dwSize))==ERROR_SUCCESS)
        res=true;
    }
    else 
      if (error==ERROR_FILE_NOT_FOUND)
        res=true;
  }  

  if (!res)
    report_error(hwnd,error,"Error removing notification entry");

  return res;
}


bool InstallLog(HWND hwnd){

  HKEY hk; 
  DWORD dwData; 
  DWORD error;
  UCHAR szBuf[80]; 
  bool res=false;
 
  if ((error=RegCreateKeyEx(HKEY_LOCAL_MACHINE,LogSubKeyPath,0,NULL,REG_OPTION_NON_VOLATILE,
    KEY_ALL_ACCESS,NULL,&hk,&dwData))==ERROR_SUCCESS){
    
    strcpy((char*)szBuf,"%SystemRoot%\\System32\\DictFilt.dll"); 
    if ((error=RegSetValueEx(hk,"EventMessageFile",0,REG_EXPAND_SZ,(LPBYTE)szBuf,
      strlen((char*)szBuf)+1))==ERROR_SUCCESS){
      
      dwData=EVENTLOG_ERROR_TYPE; 
      if ((error=RegSetValueEx(hk,"TypesSupported",0,REG_DWORD,(LPBYTE)&dwData,sizeof(DWORD)))==ERROR_SUCCESS) 
        res=true;
    }
  }
  if (!res)
    report_error(hwnd,error,"Error installing event log entry");

  return res;
}

bool RemoveLog(HWND hwnd){
  
  bool res=false;
  DWORD error;
  
  if ((error=RegDeleteKey(HKEY_LOCAL_MACHINE,LogSubKeyPath))==ERROR_SUCCESS)
    res=true;

  if (!res)
    report_error(hwnd,error,"Error removing event log entry");
  
  return res;
}


#define CAPCHAR(c) ( (c>='a')&&(c<='z') ?  c-('a'-'A'):c)
static int StrIcmp(const char* str1, const char* str2){

  size_t len=strlen(str1);
  int res=0;

  if (strlen(str2)==len){
    for (size_t i=0; i<len; i++)
      if (CAPCHAR(str1[i])!=CAPCHAR(str2[i])){
        res=-1;
        break;
      }
  }
  else res=-1;

  return res;
}

int get_entry_idx(const char *entry,const char *buf){
  int i=0;
  
  while (*(buf+i)){
    // !!!!
    if (!StrIcmp(entry,buf+i))
      break;
    i+=strlen(buf+i)+1;
  }
  return *(buf+i) ? i:-1;
}

int add_to_list(const char *entry,char *val,int size){

  if (get_entry_idx(entry,val)==-1){
    strcpy(val+size-1,entry);
    size+=strlen(entry)+1;
    val[size-1]=0;
  }
  return size;
}

int remove_from_list(const char* entry,char *val,int size){

  int idx;
  if ((idx=get_entry_idx(entry,val))!=-1){
    MoveMemory(val+idx,val+idx+strlen(entry)+1,size-idx-strlen(entry));
    size-=strlen(entry)+1;
  }
  return size ? size:1;
}

void report_error(HWND hwnd,DWORD error,const char* mess){
  char buf[255];
  char num[10];

  //sprintf(buf,"%s (%d)",mess,error);
  strcpy(buf,mess); strcat(buf," (");
  strcat(buf,itoa(error,num,10)); strcat(buf,")");

  ::MessageBox(hwnd,buf,"Dictionary filter",MB_OK|MB_ICONEXCLAMATION);  
}